/*=====================================================================
  
  This file is part of the Autodesk PLM 360 API Code Samples.

  Copyright (C) Autodesk Inc.  All rights reserved.

THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
PARTICULAR PURPOSE.
=====================================================================*/

package com.autodesk.plm360.photoAttach;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.lang.reflect.Type;
import java.nio.charset.Charset;
import java.util.ArrayList;

import javax.ws.rs.core.MediaType;

import org.apache.http.HttpException;
import org.apache.http.HttpResponse;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.mime.MultipartEntity;
import org.apache.http.entity.mime.content.ByteArrayBody;
import org.apache.http.entity.mime.content.StringBody;

import com.autodesk.plm360.photoAttach.payloads.File;
import com.autodesk.plm360.photoAttach.payloads.FileUploadRequest;
import com.autodesk.plm360.photoAttach.payloads.Item;
import com.autodesk.plm360.photoAttach.payloads.PagedCollection;
import com.google.gson.Gson;
import com.google.gson.reflect.TypeToken;

import android.os.AsyncTask;
import android.os.Bundle;
import android.provider.MediaStore;
import android.app.ListActivity;
import android.content.Intent;
import android.graphics.Bitmap;
import android.graphics.Bitmap.CompressFormat;
import android.view.Menu;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.ListAdapter;
import android.widget.AdapterView.OnItemClickListener;

public class ItemBrowseActivity extends ListActivity {
	
	public static String TENANT_NAME_KEY = "TenantName"; 
	public static String WORKSPACE_ID_KEY = "WorkspaceId"; 
	
	private Item selectedItem;
	private String m_tenantName;
	private int m_workspaceId;
	private ArrayList<Exception> m_exceptions;

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);

		setContentView(R.layout.activity_item_browse);
		
		getListView().setClickable(true);
		getListView().setOnItemClickListener(myClickListener);
		
		m_tenantName = this.getIntent().getStringExtra(TENANT_NAME_KEY);
		m_workspaceId = this.getIntent().getIntExtra(WORKSPACE_ID_KEY, 0);
		m_exceptions = new ArrayList<Exception>();
		
		GetItemsTask task = new GetItemsTask();
		
		PagedCollection<Item> items = null;
		try
		{
			items = task.execute().get();
		}
		catch (Exception ex)
    	{
			m_exceptions.add(ex);
    	}
		
		if (!m_exceptions.isEmpty()) {
			Util.showError(m_exceptions.get(0).getMessage(), this, true);
			//this.finish();
			return;
		}
		
		if (items != null && items.getElements() != null && items.getElements().size() > 0)
		{
		
			Item [] itemArray = items.getElements().toArray(new Item[0]);
			ListAdapter adapter = new ArrayAdapter<Item>(
					this, android.R.layout.simple_list_item_1, itemArray);
			
			setListAdapter(adapter);
		}
	}

	@Override
	protected void onActivityResult (int requestCode, int resultCode, Intent intent)
	{
		Bundle extras = intent.getExtras();
        Bitmap bitmap = (Bitmap)extras.get("data");
        
        ByteArrayOutputStream outStream = new ByteArrayOutputStream();
        bitmap.compress(CompressFormat.JPEG, 90, outStream);

        m_exceptions = new ArrayList<Exception>();
        UploadFileTask task = new UploadFileTask();
        try
        {
        	task.execute(selectedItem, outStream).get();
        }
        catch (Exception ex)
    	{
			m_exceptions.add(ex);
    	}
		
		if (!m_exceptions.isEmpty()) {
			Util.showError(m_exceptions.get(0).getMessage(), this, false);
		}
		else
			Util.showMessage("File uploaded", this, false);
	}
	
	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		// Inflate the menu; this adds items to the action bar if it is present.
		getMenuInflater().inflate(R.menu.item_browse, menu);
		return true;
	}
	
	public OnItemClickListener myClickListener = new AdapterView.OnItemClickListener() 
	{

	  @Override
	  public void onItemClick(AdapterView<?> arg0, View arg1, int position, long arg3) 
	  {
		  onItemClick2(position);
	  }
	};
	
	private void onItemClick2(int position)
	{
		// upload a file
		try
		{
			selectedItem = (Item)getListView().getItemAtPosition(position);
			
			Intent takePictureIntent = new Intent(MediaStore.ACTION_IMAGE_CAPTURE);
            startActivityForResult(takePictureIntent, 0);
		}
		catch (Exception ex)
    	{
			Util.showError(ex.getMessage(), this, false);
    	}
		
		return;
	}
	
	private class UploadFileTask extends AsyncTask<Object, String, Void> {
		protected Void doInBackground(Object ...inputs) {

            try 
            {
            	uploadFile((Item)inputs[0], (ByteArrayOutputStream)inputs[1]);
            }
            catch (Exception ex)
            {
            	m_exceptions.add(ex);
            }
            
            return null;
        }
	}
	
	private void uploadFile(Item item, ByteArrayOutputStream stream) 
    		throws HttpException, ClientProtocolException, IOException
    {
		FileUploadRequest uploadReq = new FileUploadRequest();
		uploadReq.setDescription("Uploaded with PLM Photo Attach sample app");
		uploadReq.setFileName("photo.jpg");
		uploadReq.setTitle("Photo");		
		
		Gson gson = new Gson();
    	//HttpPost post = new HttpPost(item.getUrl() + "/files");
		HttpPost post = new HttpPost(Util.getFilesUrl(m_tenantName, item.getWorkspaceId(), item.getId()));
    	MultipartEntity multipart = new MultipartEntity();

    	multipart.addPart("payload", new StringBody(gson.toJson(uploadReq), MediaType.APPLICATION_JSON, Charset.forName("UTF-8")));
    	multipart.addPart("file", new ByteArrayBody(stream.toByteArray(), MediaType.APPLICATION_OCTET_STREAM));
    	
    	post.setEntity(multipart);
    	HttpClient client = SessionState.instance().getHttpClient();
    	HttpResponse httpResponse = client.execute(post, SessionState.instance().getLocalContext());
    	Util.checkHttpResponse(httpResponse);

    	String outStr = Util.getJSONString(httpResponse);
    	
    	Type wkType = new TypeToken<PagedCollection<File>>() {}.getType();
    	PagedCollection<File> files = gson.fromJson(outStr, wkType);

    	if (files == null)
    		m_exceptions.add(new Exception("Error uploading file"));
    	
    	return;
    }
	
	
	private class GetItemsTask extends AsyncTask<String, String, PagedCollection<Item>> {
		protected PagedCollection<Item> doInBackground(String ...nothing) {
        	PagedCollection<Item> retVal = null;
            try 
            {
            	retVal = getItems();
            }
            catch (Exception ex)
            {
            	m_exceptions.add(ex);
            }
            
            return retVal;
        }
	}

	private PagedCollection<Item> getItems() 
    		throws HttpException, ClientProtocolException, IOException
    {
		//Workspace wk = SessionState.instance().getCurrentWorkspace();
		
    	//HttpGet get = new HttpGet(wk.getUrl() + "/items");
		HttpGet get = new HttpGet(Util.geItemsUrl(m_tenantName, m_workspaceId));
    	HttpClient client = SessionState.instance().getHttpClient();
    	HttpResponse httpResponse = client.execute(get, SessionState.instance().getLocalContext());
    	Util.checkHttpResponse(httpResponse);
    	
    	Gson gson = new Gson();
    	String outStr = Util.getJSONString(httpResponse);
    	
    	Type wkType = new TypeToken<PagedCollection<Item>>() {}.getType();
    	PagedCollection<Item> items = gson.fromJson(outStr, wkType);
	    
    	return items;
    }
	
}
