/*=====================================================================
  
  This file is part of the Autodesk PLM 360 API Code Samples.

  Copyright (C) Autodesk Inc.  All rights reserved.

THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
PARTICULAR PURPOSE.
=====================================================================*/

package com.autodesk.plm360.photoAttach;

import java.io.IOException;
import java.net.URISyntaxException;
import java.net.URL;
import java.util.ArrayList;

import javax.ws.rs.core.MediaType;

import org.apache.http.HttpException;
import org.apache.http.HttpResponse;
import org.apache.http.client.CookieStore;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.client.protocol.ClientContext;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.BasicCookieStore;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.protocol.BasicHttpContext;
import org.apache.http.protocol.HttpContext;

import com.autodesk.plm360.photoAttach.payloads.OxygenLoginCredentials;
import com.autodesk.plm360.photoAttach.payloads.Session;
import com.google.gson.Gson;
import com.sun.jersey.api.client.Client;
import com.sun.jersey.api.client.ClientResponse;
import com.sun.jersey.api.client.WebResource;
import com.sun.jersey.api.client.config.ClientConfig;
import com.sun.jersey.api.client.config.DefaultClientConfig;
import com.sun.jersey.api.json.JSONConfiguration;
import com.sun.jersey.oauth.client.OAuthClientFilter;
import com.sun.jersey.oauth.signature.OAuthParameters;
import com.sun.jersey.oauth.signature.OAuthRequest;
import com.sun.jersey.oauth.signature.OAuthSecrets;
import com.sun.jersey.oauth.signature.OAuthSignature;
import com.sun.jersey.oauth.signature.OAuthSignatureException;
import com.sun.jersey.spi.service.ServiceFinder;

import android.os.AsyncTask;
import android.os.Bundle;
import android.app.Activity;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.webkit.WebView;
import android.support.v4.app.NavUtils;

public class LoginActivity extends Activity {

	public static String TENANT_NAME_KEY = "TenantName"; 
	public static int SUCCESS_CODE = 5;

	// **** Add your Oxygen keys here ****
	static String CONSUMER_KEY =    "";
	static String CONSUMER_SECRET = "";
	public static String OXYGEN_SERVER = "https://accounts.autodesk.com";

	public static String REQUEST_TOKEN = "RequestToken";
	public static String REQUEST_SECRET = "RequestSecret";
	
	private String m_tenantName;
	private String m_requestToken;
	private String m_requestSecret;
	private Client m_jerseyClient;
	private ArrayList<Exception> m_exceptions;
	
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_login);
		// Show the Up button in the action bar.
		setupActionBar();
	
		
		if (SessionState.instance().getJerseyClient() == null) {
			ClientConfig config = new DefaultClientConfig();
	    	config.getFeatures().put(JSONConfiguration.FEATURE_POJO_MAPPING, Boolean.TRUE);
	    	Client jerseyClient = Client.create(config);
	    	SessionState.instance().setJerseyClient(jerseyClient);
		}
		m_jerseyClient = SessionState.instance().getJerseyClient();
		
    	m_tenantName = getIntent().getStringExtra(TENANT_NAME_KEY);
    	
    	m_exceptions = new ArrayList<Exception>();
		LoginTask1 task = new LoginTask1();
    	try 
    	{
    		task.execute().get();
    	}
    	catch (Exception ex)
    	{
			m_exceptions.add(ex);
    	}
		
		if (!m_exceptions.isEmpty()) {
			Log.i("LoginActivity.onCreate", m_exceptions.get(0).toString());
			Util.showError(m_exceptions.get(0).getMessage(), this, true);
			//this.finish();
			return;
		}
		
		WebView webview = (WebView)findViewById(R.id.oxygenWebView);
		webview.clearCache(true);
	    webview.getSettings().setJavaScriptEnabled(true);
	    webview.loadData("<html><body onload=\"window.scrollTo(0,1);\"></body></html>", "text/html", "UTF-8");
	    
	    AuthWebClient webclient = new AuthWebClient(m_requestToken, m_requestSecret, this);
	    webview.addJavascriptInterface(webclient, "INPUTVALUE");
	    webview.setWebViewClient(webclient);
	    
	    try
	    {
		    webview.loadUrl(OXYGEN_SERVER + 
		    		"/OAuth/Authorize" + 
		    		"?viewmode=mobile" +
		    		"&oauth_token=" + Util.urlEncodeToken(m_requestToken));
	    }
    	catch (Exception ex)
    	{
    		Log.i("LoginActivity.onCreate", ex.toString());
    		Util.showError(ex.getMessage(), this, true);
    		//this.finish();
    	}
	}

	/**
	 * Set up the {@link android.app.ActionBar}.
	 */
	private void setupActionBar() {

		getActionBar().setDisplayHomeAsUpEnabled(true);

	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		// Inflate the menu; this adds items to the action bar if it is present.
		getMenuInflater().inflate(R.menu.login, menu);
		return true;
	}

	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		switch (item.getItemId()) {
		case android.R.id.home:
			// This ID represents the Home or Up button. In the case of this
			// activity, the Up button is shown. Use NavUtils to allow users
			// to navigate up one level in the application structure. For
			// more details, see the Navigation pattern on Android Design:
			//
			// http://developer.android.com/design/patterns/navigation.html#up-vs-back
			//
			NavUtils.navigateUpFromSameTask(this);
			return true;
		}
		return super.onOptionsItemSelected(item);
	}
	
	public void onLoginCompleted(String verifier)
	{
		m_exceptions.clear();
		LoginTask2 task = new LoginTask2();
    	try 
    	{
    		task.execute(verifier).get();
    	}
    	catch (Exception ex)
    	{
			m_exceptions.add(ex);
    	}
		
		if (!m_exceptions.isEmpty()) {
			Log.i("onLoginCompleted", m_exceptions.get(0).toString());
			Util.showError(m_exceptions.get(0).getMessage(), this, true);
		}
		else {
			
			this.setResult(SUCCESS_CODE);
			this.finish();
		}
	}

	/**
	 * Get the request token and secret
	 *
	 */
	private class LoginTask1 extends AsyncTask<String, String, Void> {
		protected Void doInBackground(String ...verifier) {
            try 
            {
            	Log.i("LoginTask1", "Starting");
            	ClientResponse response = oxygenRequestToken(m_jerseyClient);
            	
            	String responseEntity = com.sun.jersey.core.impl.provider.entity.StringProvider.readFromAsString(
            			response.getEntityInputStream(), response.getType());
            	
            	m_requestToken = Util.getParameter("oauth_token", responseEntity);
            	m_requestSecret = Util.getParameter("oauth_token_secret", responseEntity);
            	
            	Log.i("LoginTask1", "Finished.  Request token=" + m_requestToken);
            }
            catch (Exception ex)
            {
            	Log.i("LoginTask1", ex.toString());
            	m_exceptions.add(ex);
            }
            
            return null;
        }
	}
	
	
	/**
	 * Get the access token and secret
	 */
	private class LoginTask2 extends AsyncTask<String, String, Void> {
		protected Void doInBackground(String ...verifier) {
            try 
            {   
            	Log.i("LoginTask2", "Starting");
            	ClientResponse response = oxygenLogin(m_jerseyClient, verifier[0]);
            	String responseEntity = com.sun.jersey.core.impl.provider.entity.StringProvider.readFromAsString(
            			response.getEntityInputStream(), response.getType());
            	
            	String accessToken = Util.getParameter("oauth_token", responseEntity);
            	String accessSecret = Util.getParameter("oauth_token_secret", responseEntity);
            	
            	Log.i("LoginTask2", "Access token=" + accessToken);
            	
            	HttpClient httpClient = new DefaultHttpClient();
            	CookieStore cookieStore = new BasicCookieStore();
            	HttpContext localContext = new BasicHttpContext();
            	localContext.setAttribute(ClientContext.COOKIE_STORE, cookieStore);
            	
            	SessionState state = SessionState.instance();
            	state.setHttpClient(httpClient);
            	state.setLocalContext(localContext);
            	HttpResponse httpResponse = plmLogin(accessToken, accessSecret);
            	
            	Util.checkHttpResponse(httpResponse);
            	Log.i("LoginTask2", "Finished");
            	return null;
            }
            catch (Exception ex)
            {
            	Log.i("LoginTask2", ex.toString());
            	m_exceptions.add(ex);
            }
            
            return null;
        }
	
		private HttpResponse plmLogin(String accessToken, String accessTokenSecret) 
	    		throws HttpException, IOException, OAuthSignatureException, URISyntaxException {
			Log.i("plmLogin", "Starting");
			
	    	OAuthParameters oauthParams = new OAuthParameters()
				.consumerKey(CONSUMER_KEY)
				.signatureMethod("HMAC-SHA1")
				.version("1.0")
				.timestamp()
				.nonce();
	    	
			oauthParams = oauthParams.token(accessToken);
			OAuthSecrets oauthSecrets = new OAuthSecrets()
				.consumerSecret(CONSUMER_SECRET)
				.tokenSecret(accessTokenSecret);

			URL loginUrl = new URL(Util.getPLMLoginUrl(m_tenantName));
			OAuthRequest request = new OAuthRequestImpl(loginUrl, "POST");
			OAuthSignature.sign(request, oauthParams, oauthSecrets);
			String authStr = request.getHeaderValues(OAuthParameters.AUTHORIZATION_HEADER).get(0);

			OxygenLoginCredentials credentials = new OxygenLoginCredentials();
		    credentials.setCustomerId(m_tenantName);
		    credentials.setValidation(authStr);
		    
		    Gson gson = new Gson();
		    String inStr = gson.toJson(credentials);

		    HttpPost post = new HttpPost(loginUrl.toURI() );
		    StringEntity entity = new StringEntity(inStr);
		    entity.setContentType(MediaType.APPLICATION_JSON);
		    post.setEntity(entity);
		    
		    HttpClient httpClient = SessionState.instance().getHttpClient();
		    
		    HttpResponse httpResponse = httpClient.execute(post, SessionState.instance().getLocalContext());
		    Util.checkHttpResponse(httpResponse);
		    
		    String outStr = Util.getJSONString(httpResponse);
		    Session session = gson.fromJson(outStr, Session.class);
		    
		    if (session == null)
		    	throw new HttpException("Invalid PLM response");
		    
		    Log.i("plmLogin", "Finished");
		    return httpResponse;   
		}
	}
	
	@SuppressWarnings("rawtypes")
	private ClientResponse oxygenRequestToken(Client client) throws HttpException {
		Log.i("oxygenRequestToken", "Starting");
    	ServiceFinder.setIteratorProvider(new ServiceProviderMappings());

    	OAuthParameters oauthParams = new OAuthParameters()
			.consumerKey(CONSUMER_KEY)
			.signatureMethod("HMAC-SHA1")
			.version("1.0")
			.timestamp()
			.nonce().
			callback("oob");
    	
    	OAuthSecrets oauthSecrets = new OAuthSecrets().consumerSecret(CONSUMER_SECRET);
    	OAuthClientFilter filter = new OAuthClientFilter(client.getProviders(), oauthParams, oauthSecrets);
    	
    	WebResource resource = client.resource(OXYGEN_SERVER + "/OAuth/RequestToken");
    	resource.addFilter(filter);
    	ClientResponse response = resource.get(ClientResponse.class);
    	
    	if (response.getStatus() < 200 || response.getStatus() >=300) 
    		throw new HttpException("Error logging in to Oxygen; HTTP code = " + response.getStatus());
    	
    	Log.i("oxygenRequestToken", "Finished");
    	return response;
    }
	
	@SuppressWarnings("rawtypes")
	private ClientResponse oxygenLogin(Client client, String verifier) throws HttpException
    {
		Log.i("oxygenLogin", "Starting");
    	ServiceFinder.setIteratorProvider(new ServiceProviderMappings());

    	OAuthParameters oauthParams = new OAuthParameters()
			.consumerKey(CONSUMER_KEY)
			.signatureMethod("HMAC-SHA1")
			.version("1.0")
			.timestamp()
			.nonce()
			.verifier(verifier);
    	
    	oauthParams.put("oauth_token", m_requestToken);
    	OAuthSecrets oauthSecrets = new OAuthSecrets().consumerSecret(CONSUMER_SECRET);
    	oauthSecrets.tokenSecret(m_requestSecret);
    	OAuthClientFilter filter = new OAuthClientFilter(client.getProviders(), oauthParams, oauthSecrets);
    	
    	WebResource resource = client.resource(OXYGEN_SERVER + "/OAuth/AccessToken");
    	resource.addFilter(filter);
    	ClientResponse response = resource.get(ClientResponse.class);
    	
    	if (response.getStatus() < 200 || response.getStatus() >=300) 
    		throw new HttpException("Error logging in to Oxygen; HTTP code = " + response.getStatus());
    	
    	Log.i("oxygenLogin", "Finished");
    	return response;
    }
}
