/*=====================================================================
  
  This file is part of the Autodesk PLM 360 API Code Samples.

  Copyright (C) Autodesk Inc.  All rights reserved.

THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
PARTICULAR PURPOSE.
=====================================================================*/

package com.autodesk.plm360.photoAttach;

import java.net.URL;
import java.util.ArrayList;

import org.apache.http.HttpResponse;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpPost;

import android.os.AsyncTask;
import android.os.Bundle;
import android.app.Activity;
import android.content.Intent;
import android.util.Log;
import android.view.Menu;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;

public class MainActivity extends Activity {

	enum LoginState {
		LoggedIn,
		LoggedOut,
		Indeterminiate
	}
	
	private LoginState m_loginState = LoginState.LoggedOut;
	private ArrayList<Exception> m_exceptions;
	
	private EditText m_tenantNameEditText;
	private Button m_loginButton;
	private String m_tenantName;

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_main);

		m_tenantNameEditText = (EditText)findViewById(R.id.tenantNameEditText);
		m_loginButton = (Button)findViewById(R.id.loginButton);
		
		// There is an exit button, which is useful for debugging purposes
		// To enable it, comment out the line that sets the button invisible.
		Button exitButton = (Button)findViewById(R.id.exitButton);
		exitButton.setVisibility(android.view.View.INVISIBLE);
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		// Inflate the menu; this adds items to the action bar if it is present.
		getMenuInflater().inflate(R.menu.main, menu);

		return true;
	}
	
	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent data) {
		super.onActivityResult(requestCode, resultCode, data);
		
		if (resultCode == LoginActivity.SUCCESS_CODE) {
			Intent intent = new Intent(this, WorkspaceBrowse.class);
			intent.putExtra(WorkspaceBrowse.TENANT_NAME_KEY, m_tenantName);
			startActivity(intent);
			m_loginState = LoginState.LoggedIn;
		}
		else
			m_loginState = LoginState.LoggedOut;

		updateView();
	}
	
	public void onClickLoginButton(View view) {
		
		if (m_loginState == LoginState.LoggedOut) {
			// ASSUMPTION: tenant names are always upper case
			m_tenantName = m_tenantNameEditText.getText().toString().toUpperCase();
			
			if (m_tenantName.isEmpty()) {
				Util.showMessage("You need to enter a tenant name", this, false);
				return;
			}
			
			Intent intent = new Intent(this.getBaseContext(), LoginActivity.class);
			intent.putExtra(LoginActivity.TENANT_NAME_KEY, m_tenantName);
			startActivityForResult(intent, LoginActivity.SUCCESS_CODE);
			
			m_loginState = LoginState.Indeterminiate;
			updateView();
		}
		else if (m_loginState == LoginState.LoggedIn) {
			logOut();
			updateView();
		}
	}	
	
	public void logOut()
	{
		m_exceptions = new ArrayList<Exception>();
		
		LogoutTask task = new LogoutTask();
    	try 
    	{
    		task.execute().get();
    	}
    	catch (Exception ex)
    	{
			m_exceptions.add(ex);
    	}
		
		if (!m_exceptions.isEmpty()) {
			Log.i("onLoginCompleted", m_exceptions.get(0).toString());
			Util.showError(m_exceptions.get(0).getMessage(), this, true);
		}
		else {
			Util.showMessage("Logout completed", this, false);
			m_tenantNameEditText.setText("");
			SessionState.instance().setHttpClient(null);
			SessionState.instance().setLocalContext(null);
			m_loginState = LoginState.LoggedOut;
		}
	}
	
	private class LogoutTask extends AsyncTask<String, String, Void> {
		protected Void doInBackground(String ...verifier) {
            try 
            {   
            	Log.i("LogoutTask", "Starting");

            	URL logutUrl = new URL(Util.getPLMLogoutUrl(m_tenantName));
            	HttpPost post = new HttpPost(logutUrl.toURI());
            	
            	HttpClient httpClient = SessionState.instance().getHttpClient();
     		    HttpResponse httpResponse = httpClient.execute(post, SessionState.instance().getLocalContext());
     		    Util.checkHttpResponse(httpResponse);
            	
            	Log.i("LogoutTask", "Finished");
            	return null;
            }
            catch (Exception ex)
            {
            	Log.i("LogoutTask", ex.toString());
            	m_exceptions.add(ex);
            }
            
            return null;
        }
	}
	
	public void onClickExitButton(View view) {
		finish();
    	android.os.Process.killProcess(android.os.Process.myPid());
    	super.onDestroy();
	}
	
	
	private void updateView()
	{
		if (m_loginState == LoginState.LoggedIn) {
			m_loginButton.setEnabled(true);
			m_loginButton.setText(R.string.log_out);
		}
		else if (m_loginState == LoginState.LoggedOut) {
			m_loginButton.setEnabled(true);
			m_loginButton.setText(R.string.log_in);
		}
		else {
			m_loginButton.setEnabled(false);
		}
	
	}

}
