/*=====================================================================
  
  This file is part of the Autodesk PLM 360 API Code Samples.

  Copyright (C) Autodesk Inc.  All rights reserved.

THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
PARTICULAR PURPOSE.
=====================================================================*/

package com.autodesk.plm360.photoAttach;

import java.io.UnsupportedEncodingException;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLDecoder;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;

import com.sun.jersey.oauth.signature.OAuthRequest;

class OAuthRequestImpl implements OAuthRequest {
	private URL requestUrl;
	private String requestMethod;
	private Map<String, List<String>> requestParameters;
	private Map<String, List<String>> requestHeaders;
	
	public OAuthRequestImpl(URL requestUrl, String requestMethod) throws MalformedURLException, UnsupportedEncodingException {
		this.requestUrl = requestUrl;
		this.requestMethod = requestMethod;
		this.requestHeaders = new HashMap<String, List<String>>();
		this.requestParameters = getUrlParameters(this.requestUrl.toString());
	}
	
	@Override
	public URL getRequestURL() {
		return requestUrl;
	}
	
	@Override
	public String getRequestMethod() {
		return requestMethod;
	}
	
	@Override
	public List<String> getParameterValues(String name) {
		return requestParameters.get(name);
	}
	
	@Override
	public Set<String> getParameterNames() {
		return requestParameters.keySet();
	}
	
	@Override
	public List<String> getHeaderValues(String name) {
		return requestHeaders.get(name);
	}
	
	@Override
	public void addHeaderValue(String name, String value)
			throws IllegalStateException {
		List<String> headerValues = requestHeaders.get(name);
		if (headerValues == null) {
			headerValues = new LinkedList<String>();
			requestHeaders.put(name, headerValues);
		}
		
		headerValues.add(value);
	}
	
	private static Map<String, List<String>> getUrlParameters(String url)
	        throws UnsupportedEncodingException {
	    Map<String, List<String>> params = new HashMap<String, List<String>>();
	    String[] urlParts = url.split("\\?");
	    if (urlParts.length > 1) {
	        String query = urlParts[1];
	        for (String param : query.split("&")) {
	            String pair[] = param.split("=");
	            String key = URLDecoder.decode(pair[0], "UTF-8");
	            String value = "";
	            if (pair.length > 1) {
	                value = URLDecoder.decode(pair[1], "UTF-8");
	            }
	            List<String> values = params.get(key);
	            if (values == null) {
	                values = new ArrayList<String>();
	                params.put(key, values);
	            }
	            values.add(value);
	        }
	    }
	    return params;
	}
}
