/*=====================================================================
  
  This file is part of the Autodesk PLM 360 API Code Samples.

  Copyright (C) Autodesk Inc.  All rights reserved.

THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
PARTICULAR PURPOSE.
=====================================================================*/

package com.autodesk.plm360.photoAttach;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.UnsupportedEncodingException;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLDecoder;
import java.net.URLEncoder;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.http.HttpException;
import org.apache.http.HttpResponse;

import android.app.Activity;
import android.app.AlertDialog;
import android.content.DialogInterface;

public class Util {
	private static String PLM_SERVER_FORMAT = "https://%1$s.autodeskplm360.net";
	
	
	public static String getPLMLoginUrl(String tenantName)
	{
		String hostname = String.format(PLM_SERVER_FORMAT, tenantName);
		return hostname + "/api/v2/authentication/oxygen-login";
	}
	
	public static String getPLMLogoutUrl(String tenantName)
	{
		String hostname = String.format(PLM_SERVER_FORMAT, tenantName);
		return hostname + "/api/v2/authentication/logout";
	}
	
	public static String getWorkspacesUrl(String tenantName)
	{
		String hostname = String.format(PLM_SERVER_FORMAT, tenantName);
		return hostname + "/api/v2/workspaces";
	}
	
	public static String geItemsUrl(String tenantName, Integer workspaceId)
	{
		return getWorkspacesUrl(tenantName) + "/" + workspaceId + "/items";
	}
	
	public static String getFilesUrl(String tenantName, Integer workspaceId, Integer itemId) {
		return geItemsUrl(tenantName, workspaceId) + "/" + itemId + "/files";
	}
	
	public static String urlEncodeToken(String token) throws UnsupportedEncodingException
	{
		token = URLEncoder.encode(token, "UTF-8");
		token = token.replaceAll(Pattern.quote("*"), "%2A");
		token = token.replaceAll(Pattern.quote("+"), "%20");
		token = token.replaceAll(Pattern.quote("%7E"), "~");
		
		return token;
	}
	
	public static void checkHttpResponse(HttpResponse httpResponse) throws HttpException
    {
    	if (httpResponse.getStatusLine().getStatusCode() < 200 || httpResponse.getStatusLine().getStatusCode() >=300) 
	    {
    		String outStr = "Unknown HTTP error";

    		try {
				String json = getJSONString(httpResponse);
				outStr = "PLM 360 error: " + json;
			} catch (IOException e) {
				outStr = e.getMessage();
			}

    		throw new HttpException(outStr);
	    }
    }
    
    public static String getJSONString(HttpResponse httpResponse) throws IOException
    {
    	BufferedReader rd = new BufferedReader(new InputStreamReader(httpResponse.getEntity().getContent()));
        
    	StringBuffer buffer = new StringBuffer();
    	String line = "";
        while ((line = rd.readLine()) != null) {
           buffer.append(line);
        }
        
        return buffer.toString();
    }
    
    
    public static String getParameter(String name, String input) throws UnsupportedEncodingException {
		// name=value pattern
    	final Pattern regexPattern = Pattern.compile(String.format(".*%s=([^=&]+).*", name), Pattern.CASE_INSENSITIVE);
		final Matcher matcher = regexPattern.matcher(input);
    	
       	return matcher.matches() ? URLDecoder.decode(matcher.group(1), "UTF-8") : null;
	}
    
   
    
    public static void showError(String errorMessage, final Activity activity, final boolean exitActivity) {
    	showMessage(R.string.error + ": " + errorMessage, activity, exitActivity);
    }
    
    public static void showMessage(String message, final Activity activity, final boolean exitActivity) {
    	AlertDialog.Builder builder = new AlertDialog.Builder(activity);
    	builder.setMessage(message);
    	builder.setPositiveButton(R.string.ok, new DialogInterface.OnClickListener() {
            @Override
            public void onClick(DialogInterface dialog, int id) {
            	if (exitActivity)
            		activity.finish();
            }});
    	builder.show();
    }
}
