/*=====================================================================
  
  This file is part of the Autodesk PLM 360 API Code Samples.

  Copyright (C) Autodesk Inc.  All rights reserved.

THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
PARTICULAR PURPOSE.
=====================================================================*/

package com.autodesk.plm360.photoAttach;

import java.io.IOException;
import java.lang.reflect.Type;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.apache.http.HttpException;
import org.apache.http.HttpResponse;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.CookieStore;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.protocol.ClientContext;
import org.apache.http.impl.client.BasicCookieStore;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.protocol.BasicHttpContext;
import org.apache.http.protocol.HttpContext;

import com.autodesk.plm360.photoAttach.payloads.PagedCollection;
import com.autodesk.plm360.photoAttach.payloads.Workspace;
import com.google.gson.Gson;
import com.google.gson.reflect.TypeToken;
import com.sun.jersey.api.client.Client;
import com.sun.jersey.api.client.ClientResponse;
import com.sun.jersey.api.client.config.ClientConfig;
import com.sun.jersey.api.client.config.DefaultClientConfig;
import com.sun.jersey.api.json.JSONConfiguration;

import android.os.AsyncTask;
import android.os.Bundle;
import android.app.Activity;
import android.app.ListActivity;
import android.app.LoaderManager;
import android.content.Intent;
import android.content.Loader;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.ArrayAdapter;
import android.widget.EditText;
import android.widget.ListAdapter;
import android.widget.ListView;
import android.widget.SimpleAdapter;
import android.widget.TextView;
import android.support.v4.app.NavUtils;

public class WorkspaceBrowse extends ListActivity{
	public static String TENANT_NAME_KEY = "TenantName"; 
	
	private String m_tenantName;
	private ArrayList<Exception> m_exceptions;

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_workspace_browse);
		// Show the Up button in the action bar.
		setupActionBar();

		m_tenantName = this.getIntent().getStringExtra(TENANT_NAME_KEY);
		
		getListView().setClickable(true);
		getListView().setOnItemClickListener(myClickListener);
		
		m_exceptions = new ArrayList<Exception>();
		GetWorkspacesTask task = new GetWorkspacesTask();
		
		PagedCollection<Workspace> workspaces = null;
		try
		{
			workspaces = task.execute().get();
		}
		catch (Exception ex)
    	{
			m_exceptions.add(ex);
    	}
		
		if (!m_exceptions.isEmpty()) {
			Util.showError(m_exceptions.get(0).getMessage(), this, true);
			//this.finish();
			return;
		}
		
		if (workspaces != null && workspaces.getElements() != null && workspaces.getElements().size() > 0)
		{
		
			Workspace [] wkArray = workspaces.getElements().toArray(new Workspace[0]);
			ListAdapter adapter = new ArrayAdapter<Workspace>(
					this, android.R.layout.simple_list_item_1, wkArray);
			
			setListAdapter(adapter);
		}
	}
	
	public OnItemClickListener myClickListener = new AdapterView.OnItemClickListener() 
	{

	  @Override
	  public void onItemClick(AdapterView<?> arg0, View arg1, int position, long arg3) 
	  {
		  onItemClick2(position);
	  }
	};
	
	private void onItemClick2(int position)
	{
		Workspace workspace = (Workspace)getListView().getItemAtPosition(position);
		//SessionState.instance().setCurrentWorkspace(workspace);
		
		Intent intent = new Intent(this, ItemBrowseActivity.class);
		intent.putExtra(ItemBrowseActivity.TENANT_NAME_KEY, m_tenantName);
		intent.putExtra(ItemBrowseActivity.WORKSPACE_ID_KEY, workspace.getId());
		
		startActivity(intent);
		return;
	}

	/**
	 * Set up the {@link android.app.ActionBar}.
	 */
	private void setupActionBar() {

		getActionBar().setDisplayHomeAsUpEnabled(true);

	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		// Inflate the menu; this adds items to the action bar if it is present.
		getMenuInflater().inflate(R.menu.workspace_browse, menu);
		return true;
	}

	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		switch (item.getItemId()) {
		case android.R.id.home:
			// This ID represents the Home or Up button. In the case of this
			// activity, the Up button is shown. Use NavUtils to allow users
			// to navigate up one level in the application structure. For
			// more details, see the Navigation pattern on Android Design:
			//
			// http://developer.android.com/design/patterns/navigation.html#up-vs-back
			//
			NavUtils.navigateUpFromSameTask(this);
			return true;
		}
		return super.onOptionsItemSelected(item);
	}

	private class GetWorkspacesTask extends AsyncTask<String, String, PagedCollection<Workspace>> {
		protected PagedCollection<Workspace> doInBackground(String ...nothing) {
        	
        	PagedCollection<Workspace> retVal = null;
            try 
            {
            	retVal = getAllWorkspaces();
            }
            catch (Exception ex)
            {
            	m_exceptions.add(ex);
            }
            
            return retVal;
        }
	}
	
	private PagedCollection<Workspace> getAllWorkspaces() 
    		throws HttpException, ClientProtocolException, IOException
    {
    	HttpGet get = new HttpGet(Util.getWorkspacesUrl(m_tenantName));
    	HttpClient client = SessionState.instance().getHttpClient();
    	HttpResponse httpResponse = client.execute(get, SessionState.instance().getLocalContext());
    	Util.checkHttpResponse(httpResponse);
    	
    	Gson gson = new Gson();
    	String outStr = Util.getJSONString(httpResponse);
    	
    	Type wkType = new TypeToken<PagedCollection<Workspace>>() {}.getType();
    	PagedCollection<Workspace> workspaces = gson.fromJson(outStr, wkType);
	    
    	return workspaces;
    }
}
